@@ -58,14 +58,21 @@ coef.boot_test_mediation <- function(object, parm = NULL,
                                      type = c("boot", "data"),
                                      ...) {
   # initializations
+  x <- object$fit$x
+  m <- object$fit$m
+  nr_indirect <- length(x) * length(m)
   type <- match.arg(type)
   # extract effects (including indirect effect)
   if(type == "boot") {
     # construct vector of bootstrap estimates
-    coef <- c(object$a, object$b, object$direct, object$total, object$ab)
-    names(coef) <- get_effect_names(object$fit$m, indirect = TRUE)
+    ab <- object$ab
+    coef <- c(object$a, object$b, object$direct, object$total, ab)
+    # add coefficient names
+    if (nr_indirect == 1L) indirect_names <- "ab"
+    else indirect_names <- paste("ab", names(ab), sep = "_")
+    names(coef) <- c(get_effect_names(x, m), indirect_names)
     # if requested, take subset of effects
-    if(!is.null(parm))  coef <- coef[parm]
+    if (!is.null(parm))  coef <- coef[parm]
   } else coef <- coef(object$fit, parm = parm, ...)
   # return effects
   coef
@@ -77,20 +84,43 @@ coef.boot_test_mediation <- function(object, parm = NULL,
 #' @export
 
 coef.fit_mediation <- function(object, parm = NULL, ...) {
+  # initializations
+  x <- object$x
+  m <- object$m
+  nr_indirect <- length(x) * length(m)
   # extract effects
-  coef <- c(object$a, object$b, object$direct, object$total, object$ab)
-  names(coef) <- get_effect_names(object$m, indirect = TRUE)
+  ab <- object$ab
+  coef <- c(object$a, object$b, object$direct, object$total, ab)
+  # add coefficient names
+  if (nr_indirect == 1L) indirect_names <- "ab"
+  else indirect_names <- paste("ab", names(ab), sep = "_")
+  names(coef) <- c(get_effect_names(x, m), indirect_names)
   # if requested, take subset of effects
-  if(!is.null(parm))  coef <- coef[parm]
+  if (!is.null(parm))  coef <- coef[parm]
   coef
 }
 
 
 # utility function to get names of coefficients
-get_effect_names <- function(m, sep = "_", indirect = FALSE) {
-  if(length(m) == 1L) c("a", "b", "Direct", "Total", if (indirect) "ab")
-  else {
-    c(paste("a", m, sep = sep), paste("b", m, sep = sep), "Direct", "Total",
-      if (indirect) paste("ab", c("Total", m), sep = sep))
+get_effect_names <- function(x, m, sep = "_") {
+  # initializations
+  p_x <- length(x)
+  p_m <- length(m)
+  # construct names
+  if (p_x == 1L) {
+    if (p_m == 1L) c("a", "b", "Direct", "Total")
+    else {
+      c(paste("a", m, sep = sep), paste("b", m, sep = sep), "Direct", "Total")
+    }
+  } else {
+    if (p_m == 1L) {
+      c(paste("a", x, sep = sep), "b", paste("Direct", x, sep = sep),
+        paste("Total", x, sep = sep))
+    } else {
+      c(paste("a", sapply(m, paste, x, sep = "."), sep = sep),
+        paste("b", m, sep = sep),
+        paste("Direct", x, sep = sep),
+        paste("Total", x, sep = sep))
+    }
   }
 }
