@@ -51,8 +51,8 @@
 #' \code{\link{covariates}()}.
 #' @param data  for the \code{formula} method, a data frame containing the
 #' variables.
-#' @param x  a character string, an integer or a logical vector specifying the
-#' column of \code{object} containing the independent variable.
+#' @param x  a character, integer or logical vector specifying the columns of
+#' \code{object} containing the independent variables.
 #' @param y  a character string, an integer or a logical vector specifying the
 #' column of \code{object} containing the dependent variable.
 #' @param m  a character, integer or logical vector specifying the columns of
@@ -64,8 +64,8 @@
 #' estimation.  Possible values are \code{"regression"} (the default)
 #' to estimate the effects via regressions, or \code{"covariance"} to
 #' estimate the effects via the covariance matrix.  Note that the effects are
-#' always estimated via regressions if more than one hypothesized mediator is
-#' specified or if control variables are supplied.
+#' always estimated via regressions if more than one independent variable or
+#' hypothesized mediator is specified, or if control variables are supplied.
 #' @param robust  a logical indicating whether to robustly estimate the effects
 #' (defaults to \code{TRUE}).  For estimation via regressions
 #' (\code{method = "regression"}), this can also be a character string, with
@@ -75,10 +75,18 @@
 #' used in maximum likelihood estimation of regression models.  Possible values
 #' are \code{"gaussian"} for a normal distribution (the default),
 #' \code{skewnormal} for a skew-normal distribution, \code{"student"} for
-#' Student's t distribution, \code{"skew-t"} for a skew-t distribution, or
+#' Student's t distribution, \code{"skewt"} for a skew-t distribution, or
 #' \code{"select"} to select among these four distributions via BIC (see
 #' \sQuote{Details}).  This is only relevant if \code{method = "regression"}
 #' and \code{robust = FALSE}.
+#' @param contrast  a logical indicating whether to compute pairwise contrasts
+#' of the indirect effects (defaults to \code{FALSE}).  This can also be a
+#' character string, with \code{"estimates"} for computing the pairwise
+#' differences of the indirect effects, and \code{"absolute"} for computing the
+#' pairwise differences of the absolute values of the indirect effects.  This
+#' is only relevant for models with multiple indirect effects, which are
+#' currently only implemented for estimation via regressions
+#' (\code{method = "regression"}).
 #' @param fit_yx  a logical indicating whether to fit the regression model
 #' \code{y ~ x + covariates} to estimate the total effect (the default is
 #' \code{TRUE}).  This is only relevant if \code{method = "regression"} and
@@ -87,7 +95,7 @@
 #' method.  For robust regression (\code{method = "regression"}, and
 #' \code{robust = TRUE} or \code{robust = "MM"}), a list of tuning
 #' parameters for \code{\link[robustbase]{lmrob}()} as generated by
-#' \code{\link{reg_control}()}.  For Huberized covariance matrix estimation
+#' \code{\link{reg_control}()}.  For winsorized covariance matrix estimation
 #' (\code{method = "covariance"} and \code{robust = TRUE}), a list of tuning
 #' parameters for \code{\link{cov_Huber}()} as generated by
 #' \code{\link{cov_control}()}.  No tuning parameters are necessary for median
@@ -100,29 +108,31 @@
 #' \code{"reg_fit_mediation"} if \code{method = "regression"} or
 #' \code{"cov_fit_mediation"} if \code{method = "covariance"}) with
 #' the following components:
-#' \item{a}{a numeric vector containing the point estimates of the effect of
-#' the independent variable on the proposed mediator variables.}
+#' \item{a}{a numeric vector containing the point estimates of the effects of
+#' the independent variables on the proposed mediator variables.}
 #' \item{b}{a numeric vector containing the point estimates of the direct
-#' effect of the proposed mediator variables on the dependent variable.}
-#' \item{direct}{numeric; the point estimate of the direct effect of the
-#' independent variable on the dependent variable.}
-#' \item{total}{numeric; the point estimate of the total effect of the
-#' independent variable on the dependent variable.}
+#' effects of the proposed mediator variables on the dependent variable.}
+#' \item{direct}{a numeric vector containing the point estimates of the direct
+#' effects of the independent variables on the dependent variable.}
+#' \item{total}{a numeric vector containing the point estimates of the total
+#' effects of the independent variables on the dependent variable.}
 #' \item{ab}{a numeric vector containing the point estimates of the indirect
 #' effects.}
 #' \item{fit_mx}{an object of class \code{"\link[robustbase]{lmrob}"},
-#' \code{"\link[stats]{lm}"} or \code{"lmse"} containing the estimation results
-#' from the regression of the proposed mediator variable on the independent
-#' variable, or a list of such objects in case of more than one hypothesized
-#' mediator (only \code{"reg_fit_mediation"}).}
+#' \code{"\link[quantreg]{rq}"}, \code{"\link[stats]{lm}"} or \code{"lmse"}
+#' containing the estimation results from the regression of the proposed
+#' mediator variable on the independent variables, or a list of such objects
+#' in case of more than one hypothesized mediator (only
+#' \code{"reg_fit_mediation"}).}
 #' \item{fit_ymx}{an object of class \code{"\link[robustbase]{lmrob}"},
-#' \code{"\link[stats]{lm}"} or \code{"lmse"} containing the estimation results
-#' from the regression of the dependent variable on the proposed mediator and
-#' independent variables (only \code{"reg_fit_mediation"}).}
+#' \code{"\link[quantreg]{rq}"}, \code{"\link[stats]{lm}"} or \code{"lmse"}
+#' containing the estimation results from the regression of the dependent
+#' variable on the proposed mediator and independent variables (only
+#' \code{"reg_fit_mediation"}).}
 #' \item{fit_yx}{an object of class \code{"\link[stats]{lm}"} or \code{"lmse"}
 #' containing the estimation results from the regression of the dependent
-#' variable on the independent variable (only \code{"reg_fit_mediation"} and
-#' if \code{robust = FALSE}).}
+#' variable on the independent variables (only \code{"reg_fit_mediation"}
+#' if arguments \code{robust = FALSE} and \code{fit_yx = TRUE} were used).}
 #' \item{cov}{an object of class \code{"\link{cov_Huber}"} or
 #' \code{"\link{cov_ML}"} containing the covariance matrix estimates
 #' (only \code{"cov_fit_mediation"}).}
@@ -133,6 +143,10 @@
 #' \item{robust}{either a logical indicating whether the effects were estimated
 #' robustly, or one of the character strings \code{"MM"} and \code{"median"}
 #' specifying the type of robust regressions.}
+#' \item{contrast}{either a logical indicating whether contrasts of the
+#' indirect effects were computed, or one of the character strings
+#' \code{"estimates"} and \code{"absolute"} specifying the type of contrasts
+#' of the indirect effects (only \code{"reg_fit_mediation"}).}
 #' \item{control}{a list of tuning parameters used (if applicable).}
 #'
 #' @note The formula interface is still experimental and may change in future
@@ -141,10 +155,9 @@
 #' @author Andreas Alfons
 #'
 #' @references
-#' Alfons, A., Ates, N.Y. and Groenen, P.J.F. (2018) A robust bootstrap test
-#' for mediation analysis.  \emph{ERIM Report Series in Management}, Erasmus
-#' Research Institute of Management.  URL
-#' \url{https://hdl.handle.net/1765/109594}.
+#' Alfons, A., Ates, N.Y. and Groenen, P.J.F. (2021) A robust bootstrap test
+#' for mediation analysis.  \emph{Organizational Research Methods},
+#' \doi{10.1177/1094428121999096}.
 #'
 #' Azzalini, A. and Arellano-Valle, R. B. (2013) Maximum penalized likelihood
 #' estimation for skew-normal and skew-t distributions.  \emph{Journal of
@@ -166,8 +179,11 @@
 #' @examples
 #' data("BSG2014")
 #'
-#' # to reproduce results in paper
-#' RNGversion("3.5.3")
+#' ## The results in Alfons et al. (2021) were obtained with an
+#' ## older version of the random number generator.  To reproduce
+#' ## those results, uncomment the call to RNGversion() below.
+#'
+#' # RNGversion("3.5.3")
 #' seed <- 20150601
 #'
 #' # formula interface
@@ -191,6 +207,7 @@
 #' @import boot
 #' @import robustbase
 #' @importFrom quantreg rq.fit
+#' @importFrom utils combn
 #' @export
 
 fit_mediation <- function(object, ...) UseMethod("fit_mediation")
@@ -242,14 +259,8 @@ fit_mediation.formula <- function(formula, data, ...) {
   # make sure that independent variable is specified
   if (length(mf) == (2 + have_covariates)) {
     stop("no independent variable specified in formula")
-  } else if (length(mf) > (3 + have_covariates)) {
-    stop("only one independent variable allowed in formula")
-  }
-  index_x <- setdiff(seq(2, 3 + have_covariates), c(index_m, index_covariates))
-  d <- dim(mf[[index_x]])
-  if (!is.null(d) && d[2] > 1) {
-    stop("only one independent variable allowed in formula")
   }
+  index_x <- setdiff(seq(2, length(mf)), c(index_m, index_covariates))
   x <- names(mf)[index_x]
   # prepare to rebuild data frame
   mf <- as.list(mf)
@@ -275,15 +286,16 @@ fit_mediation.formula <- function(formula, data, ...) {
 fit_mediation.default <- function(object, x, y, m, covariates = NULL,
                                   method = c("regression", "covariance"),
                                   robust = TRUE, family = "gaussian",
-                                  fit_yx = TRUE, control = NULL, ...) {
+                                  contrast = FALSE, fit_yx = TRUE,
+                                  control = NULL, ...) {
   ## initializations
   # prepare data set
   data <- as.data.frame(object)
   # check independent variable
   x <- data[, x, drop = FALSE]
   p_x <- ncol(x)
-  if (p_x != 1L) stop("exactly one independent variable required")
-  convert_x <- !is.numeric(x[, 1L])
+  if (p_x == 0L) stop("at least one independent variable required")
+  convert_x <- !all(sapply(x, is.numeric))
   # check dependent variable
   y <- data[, y, drop = FALSE]
   p_y <- ncol(y)
@@ -307,23 +319,18 @@ fit_mediation.default <- function(object, x, y, m, covariates = NULL,
   data <- cbind(x, y, m, covariates)
   # extract names
   cn <- names(data)
-  x <- cn[1L]
-  y <- cn[2L]
-  m <- cn[2L + seq_len(p_m)]
-  covariates <- cn[-(seq_len(2L + p_m))]
+  x <- cn[seq_len(p_x)]
+  y <- cn[p_x + 1L]
+  m <- cn[p_x + 1L + seq_len(p_m)]
+  covariates <- cn[-(seq_len(p_x + 1L + p_m))]
   # remove incomplete observations
   data <- data[complete.cases(data), ]
-  # if necessary, convert non-numeric independent variable
+  # if necessary, convert non-numeric independent variables
   if (convert_x) {
     # construct variables for design matrix as usual
     x <- data[, x, drop = FALSE]
-    x <- model.matrix(~ ., data = x)[, -1, drop = FALSE]
-    # check if there is still only one variable
+    x <- model.matrix(~ ., data = x)[, -1L, drop = FALSE]
     p_x <- ncol(x)
-    if (p_x != 1L) {
-      stop("currently only implemented for a numeric ",
-           "or binary independent variable")
-    }
     # replace independent variable in data frame with converted one
     data <- cbind(x, data[, c(y, m, covariates), drop = FALSE])
     # update variable name
@@ -333,7 +340,7 @@ fit_mediation.default <- function(object, x, y, m, covariates = NULL,
   if (convert_covariates) {
     # construct variables for design matrix as usual
     covariates <- data[, covariates, drop = FALSE]
-    covariates <- model.matrix(~ ., data = covariates)[, -1, drop = FALSE]
+    covariates <- model.matrix(~ ., data = covariates)[, -1L, drop = FALSE]
     # replace covariates in data frame with converted ones
     data <- cbind(data[, c(x, y, m), drop = FALSE], covariates)
     # update number of covariates and variable names
@@ -345,10 +352,11 @@ fit_mediation.default <- function(object, x, y, m, covariates = NULL,
   if (d[1L] <= d[2L]) stop("not enough observations")
   # check other arguments
   method <- match.arg(method)
-  if ((p_m > 1L || have_covariates) && method == "covariance") {
+  if ((p_x > 1L || p_m > 1L || have_covariates) && method == "covariance") {
     method <- "regression"
-    warning("covariance method not available with multiple mediators ",
-            "or any covariates; using regression method")
+    warning("covariance method not available with multiple independent ",
+            "variables, multiple mediators, or covariates; using regression ",
+            "method")
   }
   # regression method requires some more checks, covariance method is simpler
   if (method == "regression") {
@@ -367,10 +375,16 @@ fit_mediation.default <- function(object, x, y, m, covariates = NULL,
       family <- match.arg(family, choices = families)
       fit_yx <- isTRUE(fit_yx)
     }
+    # check whether to compute differences of indirect effect(s)
+    if (p_x == 1L && p_m == 1L) contrast <- FALSE
+    else if (is.logical(contrast)) {
+      contrast <- isTRUE(contrast)
+      if (contrast) contrast <- "estimates"
+    } else contrast <- match.arg(contrast, choices = c("estimates", "absolute"))
     # estimate effects
     reg_fit_mediation(data, x = x, y = y, m = m, covariates = covariates,
-                      robust = robust, family = family, fit_yx = fit_yx,
-                      control = control)
+                      robust = robust, family = family, contrast = contrast,
+                      fit_yx = fit_yx, control = control)
   } else {
     # check for robust method
     robust <- isTRUE(robust)
@@ -385,15 +399,19 @@ fit_mediation.default <- function(object, x, y, m, covariates = NULL,
 ## estimate the effects in a mediation model via regressions
 reg_fit_mediation <- function(data, x, y, m, covariates = character(),
                               robust = "MM", family = "gaussian",
-                              fit_yx = TRUE, control = reg_control()) {
-  # number of mediators
+                              contrast = FALSE, fit_yx = TRUE,
+                              control = reg_control()) {
+  # number of indirect effects
+  p_x <- length(x)
   p_m <- length(m)
+  nr_indirect <- p_x * p_m
   # construct predictor matrices for regression models
   n <- nrow(data)
   predictors_mx <- as.matrix(data[, c(x, covariates), drop = FALSE])
   predictors_ymx <- as.matrix(data[, c(m, x, covariates), drop = FALSE])
   # other initializations
   have_robust <- is.character(robust)
+  have_contrast <- is.character(contrast)
   estimate_yx <- fit_yx  # avoid name conflicts
   # compute regression models
   if (have_robust) {
@@ -476,29 +494,74 @@ reg_fit_mediation <- function(data, x, y, m, covariates = character(),
                fixed.param = selm_args$fixed.param)
     }
   }
-  # extract effects a and b
+  # extract effects a and b and compute indirect effect(s)
   if (p_m == 1L) {
-    a <- unname(coef(fit_mx)[2L])
-    b <- unname(coef(fit_ymx)[1L + seq_len(p_m)])
+    # extract effects a and b
+    if (p_x == 1L) a <- unname(coef(fit_mx)[2L])
+    else a <- coef(fit_mx)[1L + seq_len(p_x)]
+    b <- unname(coef(fit_ymx)[2L])
+    # compute indirect effect(s)
+    ab <- a * b
   } else {
-    a <- sapply(fit_mx, function(fit) unname(coef(fit)[2L]))
+    # extract effects a and b
+    if (p_x == 1L) {
+      a <- sapply(fit_mx, function(fit) unname(coef(fit)[2L]))
+    } else {
+      a <- lapply(fit_mx, function(fit) coef(fit)[1L + seq_len(p_x)])
+    }
     b <- coef(fit_ymx)[1L + seq_len(p_m)]
+    # compute indirect effect(s)
+    ab_list <- mapply(function(current_a, current_b) current_a * current_b,
+                      current_a = a, current_b = b, SIMPLIFY = FALSE)
+    # make sure we have vectors
+    if (p_x > 1L) a <- unlist(a, use.names = TRUE)
+    ab <- unlist(ab_list, use.names = TRUE)
   }
-  # compute indirect effect(s)
-  ab <- a * b
-  if (p_m > 1L) ab <- c(Total = sum(ab), ab)
-  # extract direct and total effect
-  direct <- unname(coef(fit_ymx)[2L + p_m])
+  # if applicable, compute total indirect effect and contrasts
+  if (nr_indirect > 1L) {
+    # if requested, compute contrasts
+    if (have_contrast) {
+      contrasts <- get_contrasts(ab, type = contrast)
+      n_contrasts <- length(contrasts)
+      names(contrasts) <- get_contrast_names(n_contrasts)
+    } else contrasts <- NULL
+    # compute total indirect effect
+    ab_total <- sum(ab)
+  }
+  # extract direct effect(s)
+  if (p_x == 1L) direct <- unname(coef(fit_ymx)[2L + p_m])
+  else direct <- coef(fit_ymx)[1L + p_m + seq_len(p_x)]
+  # extract total effect(s)
   if (have_robust || (family == "gaussian" && !estimate_yx)) {
-    total <- if(p_m == 1L) ab + direct else unname(ab["Total"]) + direct
-  } else if (estimate_yx) total <- unname(coef(fit_yx)[2L])
-  else total <- NA_real_
+    if (p_x == 1L) {
+      total <- if (p_m == 1L) ab + direct else ab_total + direct
+    } else {
+      if (p_m == 1) total <- ab + direct
+      else {
+        total <- sapply(x, function(current_x) {
+          current_ab <- sapply(ab_list, "[", current_x)
+          sum(current_ab) + unname(direct[current_x])
+        })
+      }
+    }
+  } else if (estimate_yx) {
+    if (p_x == 1) total <- unname(coef(fit_yx)[2L])
+    else total <- coef(fit_yx)[1L + seq_len(p_x)]
+  } else {
+    if (p_x == 1) total <- NA_real_
+    else {
+      total <- rep.int(NA_real_, p_x)
+      names(total) <- x
+    }
+  }
+  # combine total indirect effect, individual indirect effects, and contrasts
+  if (nr_indirect > 1L) ab <- c(Total = ab_total, ab, contrasts)
   # return results
   result <- list(a = a, b = b, direct = direct, total = total, ab = ab,
                  fit_mx = fit_mx, fit_ymx = fit_ymx, fit_yx = fit_yx,
                  x = x, y = y, m = m, covariates = covariates, data = data,
-                 robust = robust, family = family)
-  if(robust == "MM") result$control <- control
+                 robust = robust, family = family, contrast = contrast)
+  if (robust == "MM") result$control <- control
   class(result) <- c("reg_fit_mediation", "fit_mediation")
   result
 }
