@@ -33,7 +33,7 @@
 #' approximation (i.e., to assume a normal distribution of the corresponding
 #' effect with the standard deviation computed from the bootstrap replicates),
 #' or \code{"data"} to compute p-values via statistical theory based on the
-#' original data (e.g., based on a t-distribution the coefficients are
+#' original data (e.g., based on a t-distribution if the coefficients are
 #' estimated via regression).  Note that this is only relevant for mediation
 #' analysis via a bootstrap test, where the p-value of the indirect effect is
 #' always computed as described in \sQuote{Details}.
@@ -80,15 +80,17 @@ p_value.boot_test_mediation <- function(object, parm = NULL,
                                         type = c("boot", "data"),
                                         digits = 4L, ...) {
   # number of hypothesized mediators
-  p_m <- length(object$fit$m)
+  nr_indirect <- length(object$fit$x) * length(object$fit$m)
+  contrast <- object$fit$contrast          # only implemented for regression fit
+  have_contrast <- is.character(contrast)  # but this always works
   # p-values of other effects
   type <- match.arg(type)
-  if(type == "boot") {
+  if (type == "boot") {
     p_values <- get_p_value(object$fit, boot = object$reps)
   } else p_values <- get_p_value(object$fit)
   # add temporary NA's for p-values of indirect effect,
   # as those take longer to compute
-  if (p_m == 1) {
+  if (nr_indirect == 1L) {
     # only one mediator
     indirect_names <- "ab"
     alpha <- NA_real_
@@ -100,20 +102,40 @@ p_value.boot_test_mediation <- function(object, parm = NULL,
   names(alpha) <- indirect_names
   p_values <- c(p_values, alpha)
   # if requested, take subset of effects
-  if(!is.null(parm)) {
+  if (is.null(parm)) {
+    # p-values of all indirect effects need to be computed
+    which_names <- indirect_names
+    which_indices <- seq_along(which_names)
+  } else {
+    # take subset of p-values
     p_values <- p_values[parm]
     # check which p-values of indirect effects need to be computed
     which_names <- grep("ab", names(p_values), value = TRUE)
     which_indices <- match(which_names, indirect_names, nomatch = integer())
-  } else {
-    which_names <- indirect_names
-    which_indices <- seq_along(which_names)
+  }
+  # preparations to modify bootstrap object if contrasts are requested
+  n_ab <- 1L + nr_indirect
+  bootstrap <- object$reps
+  # if contrasts are requested, modify bootstrap object to contain only
+  # indirect effects and contrasts such that 'which_indices' correctly
+  # describes the columns containing the bootstrap replicates
+  if (have_contrast && any(which_indices > n_ab)) {
+    # list of all combinations of indices of the relevant indirect effects
+    indices_ab <- seq_len(n_ab)
+    combinations <- combn(indices_ab[-1L], 2, simplify = FALSE)
+    # modify bootstrap object to add contrasts
+    bootstrap$t0 <- c(bootstrap$t0[indices_ab],
+                      get_contrasts(bootstrap$t0, combinations,
+                                    type = contrast))
+    bootstrap$t <- cbind(bootstrap$t[, indices_ab],
+                         get_contrasts(bootstrap$t, combinations,
+                                       type = contrast))
   }
   # compute p-value of requested indirect effects as the smallest significance
   # level where 0 is not in the confidence interval
   if (length(which_names) > 0) {
     p_values[which_names] <- sapply(which_indices, function(j) {
-      p_value(object$reps, parm = j, digits = digits,
+      p_value(bootstrap, parm = j, digits = digits,
               alternative = object$alternative,
               type = object$type)
     })
@@ -121,74 +143,6 @@ p_value.boot_test_mediation <- function(object, parm = NULL,
   p_values
 }
 
-# p_value.boot_test_mediation <- function(object, digits = 4L, ...) {
-#   # number of hypothesized mediators
-#   p_m <- length(object$fit$m)
-#   # compute p-value
-#   if(p_m == 1L) {
-#     # only one mediator
-#     # set lower bound of significance level to 0
-#     lower <- 0
-#     # loop over the number of digits and determine the corresponding digit after
-#     # the comma of the p-value
-#     for (digit in seq_len(digits)) {
-#       # set step size
-#       step <- 1 / 10^digit
-#       # reset the significance level to the lower bound as we continue from there
-#       # with a smaller stepsize
-#       alpha <- lower
-#       # there is no rejection at the lower bound, so increase significance level
-#       # until there is rejection
-#       reject <- FALSE
-#       while(!reject) {
-#         # update lower bound and significance level
-#         lower <- alpha
-#         alpha <- alpha + step
-#         # retest at current significance level and extract confidence interval
-#         ci <- confint(object$reps, parm = 1L, level = 1 - alpha,
-#                       alternative = object$alternative, type = object$type)
-#         # reject if 0 is not in the confidence interval
-#         reject <- prod(ci) > 0
-#       }
-#     }
-#   } else {
-#     # multiple mediators
-#     rn <- rownames(object$ci)
-#     alpha <- sapply(seq_along(rn), function(j) {
-#       # set lower bound of significance level to 0
-#       lower <- 0
-#       # loop over the number of digits and determine the corresponding digit after
-#       # the comma of the p-value
-#       for (digit in seq_len(digits)) {
-#         # set step size
-#         step <- 1 / 10^digit
-#         # reset the significance level to the lower bound as we continue from there
-#         # with a smaller stepsize
-#         alpha_j <- lower
-#         # there is no rejection at the lower bound, so increase significance level
-#         # until there is rejection
-#         reject <- FALSE
-#         while(!reject) {
-#           # update lower bound and significance level
-#           lower <- alpha_j
-#           alpha_j <- alpha_j + step
-#           # retest at current significance level and extract confidence interval
-#           # ci <- retest(object, level = 1 - alpha)$ci[m, ]
-#           ci <- confint(object$reps, parm = j, level = 1 - alpha_j,
-#                         alternative = object$alternative, type = object$type)
-#           # reject if 0 is not in the confidence interval
-#           reject <- prod(ci) > 0
-#         }
-#       }
-#       # return p-value for current indirect effect
-#       alpha_j
-#     })
-#     names(alpha) <- rn
-#   }
-#   # return smallest significance level where 0 is not in the confidence interval
-#   alpha
-# }
-
 
 #' @rdname p_value
 #' @method p_value sobel_test_mediation
@@ -209,8 +163,6 @@ p_value.sobel_test_mediation <- function(object, parm = NULL, ...) {
   }
 }
 
-# p_value.sobel_test_mediation <- function(object, ...) object$p_value
-
 
 ## methods to extract p-values from model fits
 
@@ -298,34 +250,35 @@ get_p_value.cov_fit_mediation <- function(object, parm = NULL, boot = NULL,
 get_p_value.reg_fit_mediation <- function(object, parm = NULL, boot = NULL,
                                           ...) {
   # initializations
+  p_x <- length(object$x)
   p_m <- length(object$m)
   # extract point estimates and standard errors
   if(is.null(boot)) {
     # extract p-values from regression models
-    if(p_m == 1L) p_value_mx <- p_value(object$fit_mx, parm = 2L)
-    else p_value_mx <- sapply(object$fit_mx, p_value, parm = 2L)
-    p_value_ymx <- p_value(object$fit_ymx, parm = 1L + seq_len(p_m + 1L))
-    # compute p-valuel for total effect
-    if(is_robust(object)) {
+    if(p_m == 1L) p_value_mx <- p_value(object$fit_mx, parm = 1L + seq_len(p_x))
+    else p_value_mx <- sapply(object$fit_mx, p_value, parm = 1L + seq_len(p_x))
+    p_value_ymx <- p_value(object$fit_ymx, parm = 1L + seq_len(p_x + p_m))
+    # compute p-value for total effect
+    if(is.null(object$fit_yx)) {
       # p-value not available
-      p_value_yx <- NA_real_
+      p_value_yx <- rep.int(NA_real_, p_x)
     } else {
       # extract p-value from regression model
-      p_value_yx <- p_value(object$fit_yx, parm = 2L)
+      p_value_yx <- p_value(object$fit_yx, parm = 1L + seq_len(p_x))
     }
     # combine p-values
     p_values <- c(p_value_mx, p_value_ymx, p_value_yx)
-    names(p_values) <- get_effect_names(object$m)
+    names(p_values) <- get_effect_names(object$x, object$m)
   } else {
     # get indices of columns of bootstrap replicates that that correspond to
     # the respective models
     p_covariates <- length(object$fit$covariates)
-    index_list <- get_index_list(p_m, p_covariates)
+    index_list <- get_index_list(p_x, p_m, p_covariates)
     # the a path is the second coefficient in the model m ~ x + covariates
-    if (p_m == 1) keep_mx <- index_list$fit_mx[2L]
-    else keep_mx <- sapply(index_list$fit_mx, "[", 2L)
+    if(p_m == 1) keep_mx <- index_list$fit_mx[1L + seq_len(p_x)]
+    else keep_mx <- sapply(index_list$fit_mx, "[", 1L + seq_len(p_x))
     # keep b and c coefficients of model y ~ m + x + covariates
-    keep_ymx <- index_list$fit_ymx[1L + seq_len(p_m + 1)]
+    keep_ymx <- index_list$fit_ymx[1L + seq_len(p_x + p_m)]
     # index of total effect is stored separately in this list
     keep <- c(keep_mx, keep_ymx, index_list$total)
     # compute means, standard errors and z-statistics from bootstrap replicates
@@ -334,7 +287,7 @@ get_p_value.reg_fit_mediation <- function(object, parm = NULL, boot = NULL,
     z <- estimates / se
     # compute p-values
     p_values <- p_value_z(z)
-    names(p_values) <- get_effect_names(object$m)
+    names(p_values) <- get_effect_names(object$x, object$m)
   }
   # if requested, take subset of effects
   if(!is.null(parm)) p_values <- p_values[parm]
@@ -361,7 +314,7 @@ p_value.boot <- function(object, parm = 1L, digits = 4L,
     # there is no rejection at the lower bound, so increase significance level
     # until there is rejection
     reject <- FALSE
-    while(!reject) {
+    while (!reject) {
       # update lower bound and significance level
       lower <- alpha
       alpha <- alpha + step
