@@ -48,8 +48,8 @@ print_info.boot_test_mediation <- function(x, ...) {
     postfix <- ""
   }
   # use plural for multiple mediators
-  p_m <- length(x$fit$m)
-  plural <- if (length(p_m) == 1L) "" else "s"
+  nr_indirect <- length(x$fit$x) * length(x$fit$m)
+  plural <- if (nr_indirect == 1L) "" else "s"
   # return message
   cat(sprintf("%s test%s for indirect effect%s%s\n",
               prefix, plural, plural, postfix))
@@ -82,19 +82,28 @@ print_info.sobel_test_mediation <- function(x, ...) {
 # information on variables in summary of mediation analysis
 print_info.summary_fit_mediation <- function(x, ...) {
   # initializations
+  p_x <- length(x$x)
   p_m <- length(x$m)
+  nr_indirect <- p_x * p_m
   have_covariates <- length(x$covariates) > 0L
   # print information on variables
-  if (p_m == 1L) {
+  if (nr_indirect == 1L) {
     cat(sprintf("x = %s\n", x$x))
     cat(sprintf("y = %s\n", x$y))
     cat(sprintf("m = %s\n", x$m))
   } else {
-    width <- nchar(p_m) + 1L
-    cat(sprintf(paste0("%-", width, "s = %s\n"), "x", x$x))
+    width <- max(nchar(p_x), nchar(p_m)) + 1L
+    if (p_x == 1L) cat(sprintf(paste0("%-", width, "s = %s\n"), "x", x$x))
+    else {
+      x_labels <- paste0("x", seq_len(p_x))
+      cat(sprintf(paste0("%-", width, "s = %s\n"), x_labels, x$x), sep = "")
+    }
     cat(sprintf(paste0("%-", width, "s = %s\n"), "y", x$y))
-    m_labels <- paste0("m", seq_len(p_m))
-    cat(sprintf(paste0("%-", width, "s = %s\n"), m_labels, x$m), sep = "")
+    if (p_m == 1L) cat(sprintf(paste0("%-", width, "s = %s\n"), "m", x$m))
+    else {
+      m_labels <- paste0("m", seq_len(p_m))
+      cat(sprintf(paste0("%-", width, "s = %s\n"), m_labels, x$m), sep = "")
+    }
   }
   if (have_covariates) {
     cat("\nCovariates:\n")
@@ -112,6 +121,9 @@ print.fit_mediation <- function(x, info = TRUE, ...) {
   # print estimated effects
   cat("\nEffects:\n")
   print(coef(x), ...)
+  # print information on how contrasts are computed
+  # (if there are no contrasts, nothing is printed)
+  print_contrast_info(x, prefix = TRUE)
   # return object invisibly
   invisible(x)
 }
@@ -149,17 +161,21 @@ print.boot_test_mediation <- function(x, digits = max(3, getOption("digits")-3),
   if (isTRUE(info)) print_info(x, ...)
   # initializations
   m <- x$fit$m
-  p_m <- length(m)
-  plural <- if (p_m == 1L) "" else "s"
+  nr_indirect <- length(x$fit$x) * length(m)
+  # print indirect effects
+  plural <- if (nr_indirect == 1L) "" else "s"
   cat(sprintf("\nIndirect effect%s of x on y:\n", plural))
   # extract indirect effect
   ab <- cbind(Data = x$fit$ab, Boot = x$ab)
-  if (p_m == 1L) rownames(ab) <- m
+  if (nr_indirect == 1L) rownames(ab) <- m
   # extract confidence interval
-  ci <- if (p_m == 1L) t(x$ci) else x$ci
+  ci <- if (nr_indirect == 1L) t(x$ci) else x$ci
   colnames(ci) <- c("Lower", "Upper")
   # combine and print
   print(cbind(ab, ci), digits = digits, ...)
+  # print information on how contrasts are computed
+  # (if there are no contrasts, nothing is printed)
+  print_contrast_info(x$fit)
   # print additional information
   cat("---\nLevel of confidence: ", format(100 * x$level), " %\n", sep = "")
   cat(sprintf("\nNumber of bootstrap replicates: %d\n", x$R))
@@ -411,6 +427,27 @@ print.summary_test_mediation <- function(x, digits = max(3, getOption("digits")-
   invisible(x)
 }
 
+## internal function to print information on contrast definitions
+# This now expects an object of class "fit_mediation".  In future versions,
+# this could also be turned into a generic function if necessary.
+print_contrast_info <- function(object, prefix = FALSE, ...) {
+  # initializations
+  contrast <- object$contrast              # only implemented for regression fit
+  have_contrast <- is.character(contrast)  # but this always works
+  # if applicable, print indirect effect contrast definitions
+  if (have_contrast) {
+    # extract further information
+    x <- object$x
+    m <- object$m
+    nr_indirect <- length(x) * length(m)
+    # print information on contrast definitions
+    plural <- if (nr_indirect > 2L) "s" else ""
+    cat(sprintf("\nIndirect effect contrast definition%s:\n", plural))
+    contrast_info <- get_contrast_info(x, m, type = contrast, prefix = prefix)
+    print(contrast_info, row.names = FALSE)
+  }
+}
+
 ## internal function to print legend for significance stars
 print_legend <- function() {
   cat("---\nSignif. codes:  0", sQuote("***"), "0.001", sQuote("**"),
