@@ -59,10 +59,12 @@ test_that("arguments are correctly passed", {
   expect_identical(foo$y, "Y")
   expect_identical(foo$m, c("M1", "M2"))
   expect_identical(foo$covariates, c("C1", "C2"))
-  # robust fit
+  # nonrobust fit
   expect_false(foo$robust)
   expect_identical(foo$family, "gaussian")
   expect_null(foo$control)
+  # no contrasts
+  expect_false(foo$contrast)
 
 })
 
